<?php
/**
 * @package     Joomla.Platform
 * @subpackage  Database
 *
 * @copyright   Copyright (C) 2005 - 2012 Open Source Matters, Inc. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE
 *
 * @modification-author OBSIDEV S.A.R.L.
 */

defined('JPATH_PLATFORM') or die;

if(!class_exists('JDatabaseDriverMysqli')) {
	include_once dirname(__FILE__) . DIRECTORY_SEPARATOR . 'mysqli.php';
}

class JDatabaseDriverHikamysqli extends JDatabaseDriverMysqli
{

	public function __construct($options)
	{
		$options['debugmode'] = (isset($options['debugmode'])) ? $options['debugmode'] : 'table';
		parent::__construct($options);
	}

	public function execute()
	{
		$this->connect();

		if (!is_object($this->connection))
		{
			JLog::add(JText::sprintf('JLIB_DATABASE_QUERY_FAILED', $this->errorNum, $this->errorMsg), JLog::ERROR, 'database');
			throw new RuntimeException($this->errorMsg, $this->errorNum);
		}

		// Take a local copy so that we don't modify the original query and cause issues later
		$sql = $this->replacePrefix((string) $this->sql);
		if ($this->limit > 0 || $this->offset > 0)
		{
			$sql .= ' LIMIT ' . $this->offset . ', ' . $this->limit;
		}

		// Increment the query counter.
		$this->count++;

		// If debugging is enabled then let's log the query.
		if ($this->debug)
		{
			// Add the query to the object queue.
			$this->log[] = $sql;

			JLog::add($sql, JLog::DEBUG, 'databasequery');
		}

		// Reset the error values.
		$this->errorNum = 0;
		$this->errorMsg = '';

		// Execute the query. Error suppression is used here to prevent warnings/notices that the connection has been lost.
		$this->cursor = @mysqli_query($this->connection, $sql);

		// If an error occurred handle it.
		if (!$this->cursor)
		{
			$this->errorNum = (int) mysqli_errno($this->connection);
			$debug = debug_backtrace();
			$trace = '';
			if(empty($this->options['debugmode']))
				$this->options['debugmode'] = 'table';

			$this->errorMsg = '<span class="label label-warning">SQL Error</span> <strong>' . (string)mysqli_error($this->connection) . '</strong><pre style="margin-top:6px">' . $sql . '</pre>';
			if($this->options['debugmode'] == 'pre') {
				foreach($debug as $k => $d) {
					if($k == 0) continue;
					$trace .= str_replace(JPATH_ROOT, '', $d['file']) . ':' . $d['line'];
					if(!empty($d['class'])) {
						$trace .= ' ' . $d['class'] . '->' . $d['function'] . '()';
					} else { 
						$trace .= ' [function: ' . $d['function'].']';
					}
					$trace .= "\r\n";
				}
				$this->errorMsg .= '<pre style="padding-top:6px;">' . $trace . '</pre>';
			} else {
				foreach($debug as $k => $d) {
					if($k == 0) continue;
					$trace .= '<tr><td>'.str_replace(JPATH_ROOT, '', $d['file']) . ':' . $d['line'] . '</td>';
					if(!empty($d['class'])) {
						$trace .= '<td>' . $d['class'] . '->' . $d['function'] . '()</td>';
					} else { 
						$trace .= '<td>' . $d['function'].'</td>';
					}
					$trace .= '</tr>';
				}
				$this->errorMsg .= '<table style="margin-top:6px;" class="table table-condensed table-bordered table-hover"><thead><tr><th>File</th><th>Function</th></tr></thead><tbody>' . $trace . '</tbody></table>';
			}

			// Check if the server was disconnected.
			if (!$this->connected())
			{
				try
				{
					// Attempt to reconnect.
					$this->connection = null;
					$this->connect();
				}
				// If connect fails, ignore that exception and throw the normal exception.
				catch (RuntimeException $e)
				{
					JLog::add(JText::sprintf('JLIB_DATABASE_QUERY_FAILED', $this->errorNum, $this->errorMsg), JLog::ERROR, 'databasequery');
					throw new RuntimeException($this->errorMsg, $this->errorNum);
				}

				// Since we were able to reconnect, run the query again.
				return $this->execute();
			}
			// The server was not disconnected.
			else
			{
				JLog::add(JText::sprintf('JLIB_DATABASE_QUERY_FAILED', $this->errorNum, $this->errorMsg), JLog::ERROR, 'databasequery');
				throw new RuntimeException($this->errorMsg, $this->errorNum);
			}
		}

		return $this->cursor;
	}
}